Module: CGI::QueryExtension

Defined in:
lib/cgi/core.rb

Overview

Mixin module that provides the following:

  1. Access to the CGI environment variables as methods. See documentation to the CGI class for a list of these variables. The methods are exposed by removing the leading HTTP_ (if it exists) and downcasing the name. For example, auth_type will return the environment variable AUTH_TYPE, and accept will return the value for HTTP_ACCEPT.

  2. Access to cookies, including the cookies attribute.

  3. Access to parameters, including the params attribute, and overloading #[] to perform parameter value lookup by key.

  4. The initialize_query method, for initializing the above mechanisms, handling multipart forms, and allowing the class to be used in “offline” mode.

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#cookiesObject

Get the cookies as a hash of cookie-name=>Cookie pairs.



643
644
645
# File 'lib/cgi/core.rb', line 643

def cookies
  @cookies
end

#filesObject (readonly)

Get the uploaded files as a hash of name=>values pairs



650
651
652
# File 'lib/cgi/core.rb', line 650

def files
  @files
end

#paramsObject

Get the parameters as a hash of name=>values pairs, where values is an Array.



647
648
649
# File 'lib/cgi/core.rb', line 647

def params
  @params
end

Instance Method Details

#[](key) ⇒ Object

Get the value for the parameter with a given key.

If the parameter has multiple values, only the first will be retrieved; use #params to get the array of values.



900
901
902
903
904
905
906
907
908
909
910
911
912
913
914
915
916
# File 'lib/cgi/core.rb', line 900

def [](key)
  params = @params[key]
  return '' unless params
  value = params[0]
  if @multipart
    if value
      return value
    elsif defined? StringIO
      StringIO.new("".b)
    else
      Tempfile.new("CGI",encoding: Encoding::ASCII_8BIT)
    end
  else
    str = if value then value.dup else "" end
    str
  end
end

#create_body(is_large) ⇒ Object

:nodoc:



782
783
784
785
786
787
788
789
790
791
792
793
794
795
796
797
# File 'lib/cgi/core.rb', line 782

def create_body(is_large)  #:nodoc:
  if is_large
    require 'tempfile'
    body = Tempfile.new('CGI', encoding: Encoding::ASCII_8BIT)
  else
    begin
      require 'stringio'
      body = StringIO.new("".b)
    rescue LoadError
      require 'tempfile'
      body = Tempfile.new('CGI', encoding: Encoding::ASCII_8BIT)
    end
  end
  body.binmode if defined? body.binmode
  return body
end

#has_key?(*args) ⇒ Boolean Also known as: key?, include?

Returns true if a given query string parameter exists.

Returns:

  • (Boolean)


924
925
926
# File 'lib/cgi/core.rb', line 924

def has_key?(*args)
  @params.has_key?(*args)
end

#keys(*args) ⇒ Object

Return all query parameter names as an array of String.



919
920
921
# File 'lib/cgi/core.rb', line 919

def keys(*args)
  @params.keys(*args)
end

#multipart?Boolean

Returns whether the form contained multipart/form-data

Returns:

  • (Boolean)


892
893
894
# File 'lib/cgi/core.rb', line 892

def multipart?
  @multipart
end

Get the raw cookies as a string.



633
634
635
# File 'lib/cgi/core.rb', line 633

def raw_cookie
  env_table["HTTP_COOKIE"]
end

#raw_cookie2Object

Get the raw RFC2965 cookies as a string.



638
639
640
# File 'lib/cgi/core.rb', line 638

def raw_cookie2
  env_table["HTTP_COOKIE2"]
end

#unescape_filename?Boolean

:nodoc:

Returns:

  • (Boolean)


798
799
800
801
802
# File 'lib/cgi/core.rb', line 798

def unescape_filename?  #:nodoc:
  user_agent = $CGI_ENV['HTTP_USER_AGENT']
  return false unless user_agent
  return /Mac/i.match(user_agent) && /Mozilla/i.match(user_agent) && !/MSIE/i.match(user_agent)
end